@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2016 Stefan Krah. All rights reserved.
+ * Copyright (c) 2008-2020 Stefan Krah. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,36 +26,26 @@
  */
 
 
-#ifndef MPDECIMAL_H
-#define MPDECIMAL_H
+#ifndef LIBMPDEC_MPDECIMAL_H_
+#define LIBMPDEC_MPDECIMAL_H_
 
 
 #ifdef __cplusplus
+  #include <cinttypes>
+  #include <climits>
+  #include <cstdint>
+  #include <cstdio>
+  #include <cstdlib>
 extern "C" {
-  #ifndef __STDC_LIMIT_MACROS
-    #define __STDC_LIMIT_MACROS
-    #define MPD_CLEAR_STDC_LIMIT_MACROS
-  #endif
-#endif
-
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <limits.h>
-#include <assert.h>
-#include <stdint.h>
-#include <inttypes.h>
-
-
-#ifndef __GNUC_STDC_INLINE__
-  #define __GNUC_STDC_INLINE__ 1
-#endif
-#if defined(__GNUC__) && !defined(__INTEL_COMPILER)
-  #define UNUSED __attribute__((unused))
 #else
-  #define UNUSED
+  #include <inttypes.h>
+  #include <limits.h>
+  #include <stdint.h>
+  #include <stdio.h>
+  #include <stdlib.h>
 #endif
+
+
 #if (defined(__linux__) || defined(__FreeBSD__) || defined(__APPLE__)) && \
     defined(__GNUC__) && __GNUC__ >= 4 && !defined(__INTEL_COMPILER)
   #define MPD_PRAGMA(x) _Pragma(x)
@@ -83,10 +73,10 @@ extern "C" {
 /******************************************************************************/
 
 #define MPD_MAJOR_VERSION 2
-#define MPD_MINOR_VERSION 4
-#define MPD_MICRO_VERSION 2
+#define MPD_MINOR_VERSION 5
+#define MPD_MICRO_VERSION 0
 
-#define MPD_VERSION "2.4.2"
+#define MPD_VERSION "2.5.0"
 
 #define MPD_VERSION_HEX ((MPD_MAJOR_VERSION << 24) | \
                          (MPD_MINOR_VERSION << 16) | \
@@ -391,6 +381,7 @@ void mpd_print(const mpd_t *dec);
 
 /* assignment from a string */
 void mpd_qset_string(mpd_t *dec, const char *s, const mpd_context_t *ctx, uint32_t *status);
+void mpd_qset_string_exact(mpd_t *dec, const char *s, uint32_t *status);
 
 /* set to NaN with error flags */
 void mpd_seterror(mpd_t *result, uint32_t flags, uint32_t *status);
@@ -408,6 +399,8 @@ void mpd_qset_u32(mpd_t *result, uint32_t a, const mpd_context_t *ctx, uint32_t
 #ifndef LEGACY_COMPILER
 void mpd_qset_i64(mpd_t *result, int64_t a, const mpd_context_t *ctx, uint32_t *status);
 void mpd_qset_u64(mpd_t *result, uint64_t a, const mpd_context_t *ctx, uint32_t *status);
+void mpd_qset_i64_exact(mpd_t *result, int64_t a, uint32_t *status);
+void mpd_qset_u64_exact(mpd_t *result, uint64_t a, uint32_t *status);
 #endif
 
 /* quietly assign a C integer type to an mpd_t with a static coefficient */
@@ -435,7 +428,8 @@ void mpd_qfinalize(mpd_t *result, const mpd_context_t *ctx, uint32_t *status);
 
 const char *mpd_class(const mpd_t *a, const mpd_context_t *ctx);
 
-int mpd_qcopy(mpd_t *result, const mpd_t *a,  uint32_t *status);
+int mpd_qcopy(mpd_t *result, const mpd_t *a, uint32_t *status);
+int mpd_qcopy_cxx(mpd_t *result, const mpd_t *a);
 mpd_t *mpd_qncopy(const mpd_t *a);
 int mpd_qcopy_abs(mpd_t *result, const mpd_t *a, uint32_t *status);
 int mpd_qcopy_negate(mpd_t *result, const mpd_t *a, uint32_t *status);
@@ -689,7 +683,7 @@ mpd_uint_t mpd_lsd(mpd_uint_t word);
 mpd_ssize_t mpd_digits_to_size(mpd_ssize_t digits);
 /* number of digits in the exponent, undefined for MPD_SSIZE_MIN */
 int mpd_exp_digits(mpd_ssize_t exp);
-int mpd_iscanonical(const mpd_t *dec UNUSED);
+int mpd_iscanonical(const mpd_t *dec);
 int mpd_isfinite(const mpd_t *dec);
 int mpd_isinfinite(const mpd_t *dec);
 int mpd_isinteger(const mpd_t *dec);
@@ -798,15 +792,8 @@ int mpd_resize_zero(mpd_t *result, mpd_ssize_t size, mpd_context_t *ctx);
 
 
 #ifdef __cplusplus
-  #ifdef MPD_CLEAR_STDC_LIMIT_MACROS
-    #undef MPD_CLEAR_STDC_LIMIT_MACROS
-    #undef __STDC_LIMIT_MACROS
-  #endif
 } /* END extern "C" */
 #endif
 
 
-#endif /* MPDECIMAL_H */
-
-
-
+#endif /* LIBMPDEC_MPDECIMAL_H_ */
